import requests
import re
import random
from colorama import Fore, Style, init
from tqdm import tqdm
import os
import time

def GET_QQW(region, prefix):
    number_sg = f"{region}{prefix}"
    url = f"https://telphone.cn/prefix/{number_sg}/"
    response = requests.get(url)
    if response.status_code != 200:
        return
    html_content = response.text
    pattern = re.compile(r'<a href="/prefix/(\d{7})/">([\d]+)号段')
    matches = pattern.findall(html_content)
    numSG_list = [match[1] for match in matches]
    return numSG_list

def sanitize_regex_pattern(input_pattern: str) -> str:
    safe_pattern = re.escape(input_pattern)
    safe_pattern = safe_pattern.replace(r'\*', r'\d')
    return f'^{safe_pattern}$'

def QQW(input_pattern: str, numSG_list: list) -> list:
    regex_pattern = sanitize_regex_pattern(input_pattern)
    re.compile(regex_pattern)
    return [num for num in numSG_list if re.fullmatch(regex_pattern, num)]

def HSW(pattern):
    suffixes = [f'{i:04d}' for i in range(10000)] 
    regex_pattern = pattern.replace('x', '\\d')
    return [suffix for suffix in suffixes if re.fullmatch(regex_pattern, suffix)]

init(autoreset=True)

def colored_input(prompt):
    return input(Fore.GREEN + Style.BRIGHT + prompt + Fore.RESET)

def colored_print(message, color=Fore.CYAN, style=Style.NORMAL):
    print(style + color + message + Style.RESET_ALL)

def validate_input(prompt, pattern, error_msg):
    while True:
        try:
            user_input = colored_input(prompt).strip()
            if not re.match(pattern, user_input):
                raise ValueError
            return user_input
        except ValueError:
            colored_print(f"错误输入：{error_msg}", Fore.RED)

def main():
    phonex = validate_input(
        "请输入11位手机号（*为通配符）\n示例：138****1234\n> ",
        r"^\d{3}[0-9*]{8}$",
        "格式应为 3位数字+8位 数字/通配符。"
    )
    
    region = validate_input(
        "请输入地区名称\n示例：深圳\n> ",
        pattern=r"^[\u4E00-\u9FA5]{1,8}$",
        error_msg="请输入有效的中文地区名称。"
    )

    split_size = validate_input(
        "请输入分割手机号数量 (直接回车跳过)\n> ",
        pattern=r'^\d*$|^None$',
        error_msg="请输入有效的整数。"
    )

    start_time = time.time()
    numSG_list = GET_QQW(region, phonex[:3])
    matched_numSG = QQW(phonex[:7], numSG_list)
    matched_suffixes = HSW(phonex[7:])
    total = len(matched_numSG) * len(matched_suffixes)
    elapsed = time.time() - start_time
    os.makedirs('output', exist_ok=True)

    with open(f'output\\{region}{phonex.replace('*',"#")}.txt','w') as txt_file, open(f'output\\{region}{phonex.replace('*',"#")}.vcf','w') as vcf_file:
        for qqw in matched_numSG:
            for hsw in matched_suffixes:
                phone = qqw + hsw
                txt_file.write(f"{phone}\n")
                vcf_entry = f"BEGIN:VCARD\nVERSION:3.0\nN:;{phone};;;\nFN:{phone}\nTEL;type=CELL;type=VOICE;type=pref:{phone}\nEND:VCARD\n"
                vcf_file.write(vcf_entry)
    vcf_size = 0
    if split_size:
        split_size= int(split_size)
        phone_numbers = []
        for qqw in matched_numSG:
            for hsw in matched_suffixes:
                phone_numbers.append(qqw + hsw)
        os.makedirs(f'output\\{region}{phonex.replace('*',"#")}', exist_ok=True)
        for i in range(0, len(phone_numbers), split_size):
            with open(f'output\\{region}{phonex.replace('*',"#")}\\{i // split_size + 1}.vcf', 'w') as f:
                vcf_size += 1
                for phone in phone_numbers[i:i + split_size]:
                    vcf_entry = f"BEGIN:VCARD\nVERSION:3.0\nN:;{phone};;;\nFN:{phone}\nTEL;type=CELL;type=VOICE;type=pref:{phone}\nEND:VCARD\n"
                    f.write(vcf_entry)
    
    colored_print("\n" + "="*30, Fore.CYAN)
    colored_print(f"生成完成！耗时 {Fore.YELLOW}{elapsed:.2f}{Fore.CYAN} 秒", Fore.CYAN)
    colored_print(f"总计生成 {Fore.WHITE}{total:,}{Fore.CYAN} 个手机号码", Fore.CYAN)
    colored_print("\n", Fore.CYAN)
    if vcf_size !=0:
        colored_print(f"通讯录文件夹：{Fore.WHITE}{os.path.abspath(f'output\\{region}{phonex.replace('*',"#")}')}", Fore.CYAN)
        colored_print(f"文件数量: {Fore.WHITE}{str(vcf_size)}", Fore.CYAN)
        colored_print("\n", Fore.CYAN)
    colored_print(f"文本文件：{Fore.WHITE}{os.path.abspath(f'output\\{region}{phonex.replace('*',"#")}.txt')}", Fore.CYAN)
    colored_print(f"通讯录文件：{Fore.WHITE}{os.path.abspath(f'output\\{region}{phonex.replace('*',"#")}.vcf')}", Fore.CYAN)

if __name__ == '__main__':
    try:
        main()
    except Exception as e:
        colored_print(f"发生未预期错误：{str(e)}", Fore.RED)
        colored_print("建议操作：", Fore.RED)
        colored_print("1. 检查网络连接", Fore.RED)
        colored_print("2. 查看输入格式", Fore.RED)
        colored_print("3. 检查文件写入权限", Fore.RED)