from __future__ import annotations
from typing import List, Optional
from apipy.area_loader import AreaLoader
from apipy.date_generator import DateGenerator
from apipy.validator import calculate_check_code
from concurrent.futures import ThreadPoolExecutor
import itertools
import re
import os
import time
from datetime import datetime
from colorama import Fore, Back, Style, init


init(autoreset=True)
def colored_input(prompt):
    return input(Fore.GREEN + Style.BRIGHT + prompt + Fore.RESET + Style.RESET_ALL)
def colored_print(text, color=Fore.BLUE, style=Style.NORMAL):
    print(style + color + text + Style.RESET_ALL)
def colored_error(text):
    print(Fore.RED + Style.BRIGHT + text + Style.RESET_ALL)

class IDGenerator:
    def __init__(self, db_path='DB/area_format.db'):
        self.area_loader = AreaLoader(db_path)
    
    def generate(self, pattern: str) -> List[str]:
        if not re.match(r"^[\d*]{18}$", pattern):
            raise ValueError("身份证模式必须为18位（允许数字/*）")
        
        area_candidates = [a[0] for a in self.area_loader.get_possible_areas(pattern[:6])]
        birth_candidates = DateGenerator.generate_dates(pattern[6:14])
        seq_candidates = self._generate_sequence(pattern[14:17])
        
        valid_ids = []
        with ThreadPoolExecutor(max_workers=8) as executor:
            futures = [executor.submit(self._generate_full_id, a, b, s, pattern[17]) 
                     for a, b, s in itertools.product(area_candidates, birth_candidates, seq_candidates)]
            valid_ids = [f.result() for f in futures if f.result()]
        return valid_ids
    
    def _generate_sequence(self, seq_part: str) -> List[str]:
        return [f"{i:03d}" for i in range(1000) if all(
            c.lower() in ('*', str(i).zfill(3)[idx].lower()) for idx, c in enumerate(seq_part)
        )]
    
    def _generate_full_id(self, area: str, birth: str, seq: str, check_pattern: str) -> Optional[str]:
        pre_17 = area + birth + seq
        if len(pre_17) != 17: return None
        actual_check = calculate_check_code(pre_17)
        return pre_17 + actual_check if check_pattern.lower() in ('*', actual_check.lower()) else None
    
    def generate_with_report(self, pattern: str, output_dir="output") -> dict:
        start_time = time.time()
        os.makedirs(output_dir, exist_ok=True)
        results = self.generate(pattern)
        elapsed = time.time() - start_time

        filepath = os.path.join(output_dir, f"{pattern.replace('*',"#")}.txt")
        with open(filepath, 'w', encoding='utf-8') as f:
            f.write("\n".join(results))
        return {
            "total": len(results),
            "areas": self.area_loader.get_area_details(pattern[:6]),
            "output_path": os.path.abspath(filepath),
            "time_elapsed": elapsed
        }

def main():
    generator = IDGenerator()
    id_pattern = colored_input("请输入18位的模糊身份证, '*' 为通配符\n示例：11010119800101****\n> ").strip()
    report = generator.generate_with_report(id_pattern)
    colored_print(f"\n{'='*30}\n生成完成！耗时 {Fore.YELLOW}{report['time_elapsed']:.2f} {Fore.CYAN}秒", Fore.CYAN)
    colored_print(f"共找到 {Fore.WHITE}{report['total']} {Fore.CYAN}个有效身份证\n", Fore.CYAN)
    colored_print("匹配的地区信息：", Fore.CYAN)
    for area in report['areas']: 
        print(f"{Fore.CYAN}地区: {Fore.WHITE}{area['code']} {Fore.CYAN}| 名称: {Fore.WHITE}{area['name']}")
    colored_print(f"\n结果文件已保存至：{Fore.WHITE}{report['output_path']}", Fore.CYAN)
    colored_print(f"文件大小：{Fore.WHITE}{os.path.getsize(report['output_path'])/1024:.2f} KB", Fore.CYAN)

if __name__ == '__main__':
    try:
        main()
    except Exception as e: 
        colored_error(f"发生未预期错误：: {str(e)}")
        colored_error("建议操作：")
        colored_error("1. 检查输入格式是否为18位（例: 110105********123X）")
        colored_error("2. 确认地区代码是否有效")
        colored_error("3. 检查文件写入权限")